# import asyncio

from textual.app import App, ComposeResult
from textual.containers import Container, Vertical, VerticalScroll

# from textual.reactive import reactive
from textual.widgets import Button, Footer, Header, Static

from new_umanager.widgets.add_new_user import NewUser
from new_umanager.widgets.change_password import ChangePassword
from new_umanager.widgets.delete_user import DeleteUser
from new_umanager.widgets.renew_user import RenewUser


class UManager(App):
    BINDINGS: list[tuple] = [
        ("ctrl+q", "quit", "Quit"),
        ("ctrl+d", "toggle_dark", "Toggle Dark Mode"),
        ("A", "add_user", "New User"),
        ("P", "change_password", "Change Password"),
        ("D", "delete_user", "Delete User"),
        ("R", "renew_user", "Renew User")
    ]
    CSS_PATH: str = "umanager.css"

    def compose(self) -> ComposeResult:
        yield Header("User Manager")
        with Container(id="app-grid"):
            with VerticalScroll(id="main-menu", classes="selectors"):
                yield Button("New User", id="new-user", variant="primary")
                yield Button("Change User Password", id="change-password")
                yield Button("Delete User", id="delete-user", variant="primary")
                yield Button("Renew User", id="renew-user")
            with Vertical(id="body"):
                yield Static("Welcome to User Manager", id="mode")
        yield Footer()

    def remove_current_mode(self):
        # Remove the current mode
        self.query(".selected").remove_class("selected")
        objects = self.query("#mode")
        if objects:
            objects.remove()

    def on_button_pressed(self, event : Button.Pressed) -> None:
        button_id = event.button.id
        if button_id == "new-user":
            self.action_add_user()
        elif button_id == "change-password":
            self.action_change_password()
        elif button_id == "delete-user":
            self.action_delete_user()
        elif button_id == "renew-user":
            self.action_renew_user()

    def action_add_user(self):
        self.remove_current_mode()
        self.query("#new-user").add_class("selected")
        # Add the new mode
        new_NewUser = NewUser(id="mode")
        self.query_one("#body").mount(new_NewUser)
        new_NewUser.scroll_visible()

    def action_change_password(self):
        self.remove_current_mode()
        self.query("#change-password").add_class("selected")
        # Add the new mode
        new_NewUser = ChangePassword(id="mode")
        self.query_one("#body").mount(new_NewUser)
        new_NewUser.scroll_visible()

    def action_delete_user(self):
        self.remove_current_mode()
        self.query("#delete-user").add_class("selected")
        # Add the new mode
        new_NewUser = DeleteUser(id="mode")
        self.query_one("#body").mount(new_NewUser)
        new_NewUser.scroll_visible()

    def action_renew_user(self):
        self.remove_current_mode()
        self.query("#renew-user").add_class("selected")
        # Add the new mode
        new_NewUser = RenewUser(id="mode")
        self.query_one("#body").mount(new_NewUser)
        new_NewUser.scroll_visible()


if __name__ == "__main__":
    app = UManager()
    app.run()
