import subprocess

from textual import on
from textual.app import ComposeResult
from textual.reactive import reactive
from textual.validation import Length, Regex
from textual.widgets import Button, Input, Pretty, Static

from new_umanager.constants import *


class DeleteUser(Static):
    def compose(self) -> ComposeResult:
        yield Static("Delete User Mode", id="delete-user-mode")
        yield Input(
            placeholder="Enter LDAP Password",
            password=True,
            id="ldap-password",
            validators=[],
        )
        yield Input(
            placeholder="Enter Username",
            id="username",
            validators=[Regex(r"[a-z0-9_]+"), Length(minimum=3)],
        )
        yield Input(
            placeholder="Enter Bagde Number",
            id="badge-number",
            validators=[Regex(r"[A-Za-z0-9_]*")],
            validate_on=["submitted"],
            valid_empty=True,
        )
        yield Pretty([])
        yield Button("Delete User", id="delete-user", variant="primary")

    @on(Input.Changed)
    def show_invalid_reasons(self, event: Input.Changed) -> None:
        if not event.validation_result.is_valid:
            self.query_one(Pretty).update(event.validation_result.failure_descriptions)
        else:
            self.query_one(Pretty).update("")

    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.action_delete_user()

    def action_delete_user(self):
        # Check fields validity
        if self.ldap.value == "":
            nps.notify_confirm(LDAP_PASSWORD, WARNING)
            return
        if self.uname.value == "":
            nps.notify_confirm(INSERT_USERNAME, WARNING)
            return

        # Check if this user exists
        if ldap.userexists(self.uname.value):
            pass
        else:
            errormsg = USER + self.uname.value + USER_NOT_EXIST
            nps.notify_confirm(errormsg, WARNING, editw=1)
            return

        # Try to connect to LDAP database
        try:
            db = ldap.lcmldap(
                "ldaps://nanos8.pcteor1.mi.infn.it/",
                "cn=Manager",
                "dc=nanos8,dc=pcteor1",
                self.ldap.value,
            )
        except:
            self.query_one(Pretty).update(f"{DB_CONN_FAIL}:{WARNING}")
            self.ldap.value = None
            return

        # Ask to confirm you really want to delete this user
        dele = nps.notify_yes_no(
            CONFIRM_DEL + self.uname.value + "?", DELETE_USER, editw=2
        )
        if dele:
            dele2 = nps.notify_yes_no(
                f"{ARE_YOU_SURE}{self.uname.value}\n{IT_CANNOT_BE_UNDO}{DELETE_USER}",
            )
            if dele2:
                pass
            else:
                return
        else:
            return

        delusercmd : list[str]= f"userdel -r {self.uname.value}".split() 

        subprocess.run(delusercmd, check=True)

        db.deluser(self.uname.value)

        nps.notify_confirm(USER_DELETED, WARNING)
